//
//  ViewController.m
//  Test App
//

#import "ViewController.h"


NSString *Show_HoverView = @"SHOW";

@implementation ViewController
@synthesize effectsBar, imageView, effectsBarSegmentedControl, toolBar, changePhotoButton, imagePicker;
@synthesize imageScrollView = _imageScrollView;

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

// Used to work out the minimum zoom, called when device rotates (as aspect ratio of ScrollView changes when this happens). Could become part of NYOBetterUIScrollView but put here for now as you may not want the same behaviour I do in this regard :)
- (void)setMinimumZoomForCurrentFrame {
	UIImageView *zoomingImageView = (UIImageView *)[self.imageScrollView childView];
    
	// Work out a nice minimum zoom for the image - if it's smaller than the ScrollView then 1.0x zoom otherwise a scaled down zoom so it fits in the ScrollView entirely when zoomed out
	CGSize imageSize = zoomingImageView.image.size;
	CGSize scrollSize = self.imageScrollView.frame.size;
	CGFloat widthRatio = scrollSize.width / imageSize.width;
	CGFloat heightRatio = scrollSize.height / imageSize.height;
	CGFloat minimumZoom = MIN(1.0, (widthRatio > heightRatio) ? heightRatio : widthRatio);
	
	[self.imageScrollView setMinimumZoomScale:minimumZoom];
}


- (void)setMinimumZoomForCurrentFrameAndAnimateIfNecessary {
	BOOL wasAtMinimumZoom = NO;
    
	if(self.imageScrollView.zoomScale == self.imageScrollView.minimumZoomScale) {
		wasAtMinimumZoom = YES;
	}
	
	[self setMinimumZoomForCurrentFrame];
	
	if(wasAtMinimumZoom || self.imageScrollView.zoomScale < self.imageScrollView.minimumZoomScale) {
		[self.imageScrollView setZoomScale:self.imageScrollView.minimumZoomScale animated:YES];
	}	
}


#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString *plistPath = [documentsDirectory stringByAppendingPathComponent:@"SelectedSegmentIndex.plist"];
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSString  *normalImagePath = [documentsDirectory stringByAppendingPathComponent:@"Normal.jpg"];
    
    // Set up the custom ScrollView
	self.imageScrollView = [[NYOBetterZoomUIScrollView alloc] initWithFrame:self.view.bounds];
	[self.imageScrollView setBackgroundColor:[UIColor blackColor]];
	[self.imageScrollView setAutoresizingMask:UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight];
	[self.imageScrollView setShowsVerticalScrollIndicator:NO];
	[self.imageScrollView setShowsHorizontalScrollIndicator:NO];
	[self.imageScrollView setBouncesZoom:YES];
	[self.imageScrollView setDelegate:self];
    
    // Add gesture recognizers to ScrollView
    UITapGestureRecognizer *singleTap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleSingleTap:)];
    UITapGestureRecognizer *doubleTap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleDoubleTap:)];
    [doubleTap setNumberOfTapsRequired:2];
    [_imageScrollView addGestureRecognizer:singleTap];
    [_imageScrollView addGestureRecognizer:doubleTap];
    
    // Set up the ImageView that goes inside the scroll view
  if ([fileManager fileExistsAtPath:normalImagePath]) {
        [self setUpImageView];
    }
    
	// Finish the ScrollView setup
	[self.imageScrollView setMaximumZoomScale:2.0];
	[self setMinimumZoomForCurrentFrame];
	[self.imageScrollView setZoomScale:self.imageScrollView.minimumZoomScale animated:NO];
    [self.view addSubview:self.imageScrollView];
    
    // Load the last selected effectsBar segment
    NSMutableArray *plist = [[NSMutableArray alloc] initWithContentsOfFile:plistPath];
    NSNumber *savedSegmentIndex = [plist objectAtIndex:0];
    NSInteger segmentToLoad = [savedSegmentIndex integerValue];
    effectsBarSegmentedControl.selectedSegmentIndex = segmentToLoad;
    [self segmentedControlIndexChanged];
    
    // Move effectsBar and toolBar to on top of UIScrollView
    [self.view bringSubviewToFront:effectsBar];
    [self.view bringSubviewToFront:toolBar];
    
     // Prepare image picker
    self.imagePicker = [[UIImagePickerController alloc] init];
	self.imagePicker.allowsEditing = NO;
	self.imagePicker.delegate = self;	
    self.imagePicker.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
    
    // Hide effectsBar and toolBar from the start
    effectsBar.alpha = 0.0;
    toolBar.alpha = 0.0;
    
    // Called by MainView, when the user touches twice on the background image
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(showViewNotif:) name:Show_HoverView object:nil];
}

- (void)setUpImageView {
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString  *normalImagePath = [documentsDirectory stringByAppendingPathComponent:@"Normal.jpg"];
    UIImage *image = [[UIImage alloc] initWithContentsOfFile:normalImagePath];

    imageView = [[UIImageView alloc] initWithImage:image];
    
    [self.imageScrollView setContentSize:imageView.frame.size];
    [self.imageScrollView setChildView:imageView];
	[self.imageScrollView setMaximumZoomScale:2.0];
	[self setMinimumZoomForCurrentFrame];
    [self.imageScrollView setZoomScale:self.imageScrollView.minimumZoomScale animated:NO];  
}

- (UIView *)viewForZoomingInScrollView:(NYOBetterZoomUIScrollView *)aScrollView {
	return [aScrollView childView];
}

- (void)scrollViewDidEndZooming:(NYOBetterZoomUIScrollView *)scrollView withView:(UIView *)view atScale:(float)scale {
#ifdef DEBUG
	UIView *theView = [scrollView childView];
	NSLog(@"view frame: %@", NSStringFromCGRect(theView.frame));
	NSLog(@"view bounds: %@", NSStringFromCGRect(theView.bounds));
#endif
}

#pragma mark TapDetectingImageViewDelegate methods

- (void)handleSingleTap:(UIGestureRecognizer *)gestureRecognizer
{
    if (effectsBar.alpha == 1.0 || toolBar.alpha == 1.0) {
        [[NSNotificationCenter defaultCenter] postNotificationName:Show_HoverView object:nil];
    }
}

- (void)handleDoubleTap:(UIGestureRecognizer *)gestureRecognizer
{
    if (effectsBar.alpha == 0.0 || toolBar.alpha == 0.0) {
        [[NSNotificationCenter defaultCenter] postNotificationName:Show_HoverView object:nil];
    }
}

- (void)showHoverView:(BOOL)show
{	
	// Fade animate the view out of view by affecting its alpha
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:0.40];
    
	if (show)
	{
		effectsBar.alpha = 1.0;
        toolBar.alpha = 1.0;
    }
	else
	{
		effectsBar.alpha = 0.0;
        toolBar.alpha = 0.0;
	}
	
	[UIView commitAnimations];
}

- (void)showViewNotif:(NSNotification *)aNotification
{
	[self showHoverView:(effectsBar.alpha != 1.0)];
}

- (IBAction)changePhoto:(id)sender
{
    [self presentModalViewController:self.imagePicker animated:YES];
}

- (IBAction)segmentedControlIndexChanged
{
    // Keep track of which segment of effectsBar is selected
    int selectedSegment = ((UISegmentedControl*) effectsBarSegmentedControl).selectedSegmentIndex;
    
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString  *normalImagePath = [documentsDirectory stringByAppendingPathComponent:@"Normal.jpg"];
    NSString  *bwImagePath = [documentsDirectory stringByAppendingPathComponent:@"Bw.jpg"];
    NSString  *sepiaImagePath = [documentsDirectory stringByAppendingPathComponent:@"Sepia.jpg"];
    NSString *enhanceImagePath = [documentsDirectory stringByAppendingPathComponent:@"Enhance.jpg"];
    UIImage *imageToApplyEffect = [[UIImage alloc] initWithContentsOfFile:normalImagePath];
    NSFileManager *fileManager = [NSFileManager defaultManager];
    
    // Apply different effects to the photo depending on which effectsBar segment is selected
    if ([fileManager fileExistsAtPath:documentsDirectory]) {
        
        if (selectedSegment == 0) {
           UIImage *image = [[UIImage alloc] initWithContentsOfFile:normalImagePath];
           imageView.image = image;
        }
        
        // Grayscale effect
        else if (selectedSegment == 1) {
            if (![fileManager fileExistsAtPath:bwImagePath]){
                // Apply effect
                UIImage *imageWithEffectApplied = [imageToApplyEffect grayscale];
                imageView.image = imageWithEffectApplied;
                
                // Save image to "Documents" folder as a JPEG
                NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
                NSString *documentsDirectory = [paths objectAtIndex:0];    
                NSString  *jpgPath = [documentsDirectory stringByAppendingPathComponent:@"Bw.jpg"];
                [UIImageJPEGRepresentation(imageWithEffectApplied, 1.0) writeToFile:jpgPath atomically:YES];
            }
            else {
                imageView.image = [[UIImage alloc] initWithContentsOfFile:bwImagePath];
            }
        }
        
        // Sepia effect
        else if (selectedSegment == 2) {
            if (![fileManager fileExistsAtPath:sepiaImagePath]){
                UIImage *imageWithEffectApplied = [imageToApplyEffect sepia];
                imageView.image = imageWithEffectApplied;
                
                // Save image to "Documents" folder as a JPEG
                NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
                NSString *documentsDirectory = [paths objectAtIndex:0];    
                NSString  *jpgPath = [documentsDirectory stringByAppendingPathComponent:@"Sepia.jpg"];
                [UIImageJPEGRepresentation(imageWithEffectApplied, 1.0) writeToFile:jpgPath atomically:YES];
            }
            else {
                imageView.image = [[UIImage alloc] initWithContentsOfFile:sepiaImagePath];
            }
        }
        
        // Enhance effect
        else if (selectedSegment == 3) {
            if (![fileManager fileExistsAtPath:enhanceImagePath]){
                UIImage *imageWithEffectApplied = [imageToApplyEffect autoEnhance];
                imageView.image = imageWithEffectApplied;
                
                // Save image to "Documents" folder as a JPEG
                NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
                NSString *documentsDirectory = [paths objectAtIndex:0];    
                NSString  *jpgPath = [documentsDirectory stringByAppendingPathComponent:@"Enhance.jpg"];
                [UIImageJPEGRepresentation(imageWithEffectApplied, 1.0) writeToFile:jpgPath atomically:YES];
            }
            else {
                imageView.image = [[UIImage alloc] initWithContentsOfFile:enhanceImagePath];
            }
        }
    }
    
    //------------------------------
    // Save the selected segment of effectsBar to a plist
    // so we can load it again when the app is relaunched
    //------------------------------
    
    NSString *plistPath = [documentsDirectory stringByAppendingPathComponent:@"SelectedSegmentIndex.plist"];
    NSMutableArray *selectedSegmentIndexArray = [[NSMutableArray alloc] init];
    NSNumber *selectedSegmentForPlist = [[NSNumber alloc] initWithInt:selectedSegment];
    
    // [selectedSegmentIndexDictionary setObject:selectedSegmentForPlist forKey:@"segmentIndex"];
    [selectedSegmentIndexArray addObject:selectedSegmentForPlist];
    
    // Save plist file
    [selectedSegmentIndexArray writeToFile:plistPath atomically:YES];
}

- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingImage:(UIImage *)img editingInfo:(NSDictionary *)editInfo
{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    
    // Delete previous images
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSError *error = nil;
    NSArray *directoryContents = [fileManager contentsOfDirectoryAtPath:documentsDirectory error:&error];
    if (error == nil) {
        for (NSString *path in directoryContents) {
            NSString *fullPath = [documentsDirectory stringByAppendingPathComponent:path];
            BOOL removeSuccess = [fileManager removeItemAtPath:fullPath error:&error];
            if (!removeSuccess) {
                // Error handling
                NSLog(@"Image files weren't deleted");
            }
        }
    }
    else {
        NSLog(@"No image files to delete");
    }
    
    // Switch segmentControl back to "Normal"
    int selectedSegment;
    selectedSegment = ((UISegmentedControl*) effectsBarSegmentedControl).selectedSegmentIndex;
    [self.effectsBarSegmentedControl setSelectedSegmentIndex:0];
    
    // Save image to "Documents" folder as a JPEG
    NSString *imageName = [NSString stringWithFormat:@"Normal.jpg"];
    NSString  *jpgPath = [documentsDirectory stringByAppendingPathComponent:imageName];
    [UIImageJPEGRepresentation(img, 0.5) writeToFile:jpgPath atomically:YES];
                           
    // Assign the saved image to imageView
    [self setUpImageView];
    
    [self dismissModalViewControllerAnimated:YES];
}

- (void)viewDidUnload
{
    [super viewDidUnload];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString *jpgPath = [documentsDirectory stringByAppendingPathComponent:@"Normal.jpg"];
    NSFileManager *fileManager = [NSFileManager defaultManager];
 
    // Load the image picker if image doesn't exist
    if (![fileManager fileExistsAtPath:jpgPath]) {
        [self presentModalViewController:self.imagePicker animated:YES];
    }
}

- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return YES;
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation {
	// Aspect ratio of ScrollView has changed, need to recalculate the minimum zoom
	[self setMinimumZoomForCurrentFrameAndAnimateIfNecessary];
}
@end